@@ -103,12 +103,19 @@
 
 typedef unsigned char byte;
 
-#if defined(_MSC_VER) && !defined(MPG123_DEF_SSIZE_T)
+#if defined(_MSC_VER)
+
+// For _setmode(), at least.
+#include <io.h>
+
+#if !defined(MPG123_DEF_SSIZE_T)
 #define MPG123_DEF_SSIZE_T
 #include <stddef.h>
 typedef ptrdiff_t ssize_t;
 #endif
 
+#endif
+
 /* A safe realloc also for very old systems where realloc(NULL, size) returns NULL. */
 void *safe_realloc(void *ptr, size_t size);
 // Also freeing ptr if result is NULL. You can do
